/*
Copyright 2020 Peter Howkins

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#include "stdint.h"
#include "os.h"

// It seems that OS functions that work on too many registers end up not
// being able to inline due to complaints that the asm calls end up
// impossible
//
// possible compiler bug
//
// e.g.
//
// error: 'asm' operand has impossible constraints
//  __asm__ __volatile__("jsr $ffce" : "+Aq" (rega) : "xq"(x), "yq"(y) );
//  ^~~~~~~
//ostest.c:95:1: error: insn does not satisfy its constraints:


void
osword(uint8_t a_func, const void *xy_pointer)
{
	uint8_t regx = ((uintptr_t) xy_pointer) & 0xff;
	uint8_t regy = (((uintptr_t) xy_pointer) & 0xff00) >> 8;

	__asm__ __volatile__("jsr $fff1" : : "Aq" (a_func), "xq"(regx), "yq"(regy));
}

uint8_t
osfind(uint8_t a_operation, uint8_t x, uint8_t y)
{
	uint8_t rega = a_operation;

	__asm__ __volatile__("jsr $ffce" : "+Aq" (rega) : "xq"(x), "yq"(y) );

	return rega;
}

void
osfile(uint8_t a_action, const osfile_parameterblock *xy_pblock)
{
	uint8_t regx = ((uintptr_t) xy_pblock) & 0xff;
	uint8_t regy = (((uintptr_t) xy_pblock) & 0xff00) >> 8;

	__asm__ __volatile__("jsr $ffdd" : : "Aq" (a_action), "xq"(regx), "yq"(regy));
}

void
osgbpb(uint8_t a_action, const osgbpb_controlblock *xy_pblock)
{
	uint8_t regx = ((uintptr_t) xy_pblock) & 0xff;
	uint8_t regy = (((uintptr_t) xy_pblock) & 0xff00) >> 8;

	__asm__ __volatile__("jsr $ffd1" : : "Aq" (a_action), "xq"(regx), "yq"(regy));
}

uint8_t
osrdrm(uint8_t y_romnum, uintptr_t address)
{
	uint8_t rega = 0;

	// Set the address in the zero page values
	uint8_t *f6 = (void *) 0xf6;
	uint8_t *f7 = (void *) 0xf7;

	*f6 = address & 0xff;
	*f7 = (address & 0xff00) >> 8;

	__asm__ __volatile__("jsr $ffb9" : "+Aq" (rega) : "yq"(y_romnum) );

	return rega;
}

void
oswrsc(uint8_t a_byte, uintptr_t address)
{
	// Set the address in the zero page values
	uint8_t *d6 = (void *) 0xd6;
	uint8_t *d7 = (void *) 0xd7;

	*d6 = address & 0xff;
	*d7 = (address & 0xff00) >> 8;

	__asm__ __volatile__("jsr $ffb3" : : "Aq" (a_byte) );
}
