/*
Copyright 2020 Peter Howkins

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#ifndef STDINT____H
#define STDINT____H

/**
 * @file
 * @brief Named types for various sizes
 * 
 * C99 style header 
 */ 

/**< Main types */
typedef unsigned char uint8_t;  /**< 8 bit unsigned */
#define UINT8_MAX 255

typedef unsigned int uint16_t;  /**< 16 bit unsigned */
#define UINT16_MAX 65535

typedef unsigned long uint32_t; /**< 32 bit unsigned */
#define UINT32_MAX 4294967295

typedef signed char int8_t;     /**< 8 bit signed */
#define INT8_MAX  127
#define INT8_MIN -128

typedef int int16_t;            /**< 16 bit signed */
#define INT16_MAX  32767‬
#define INT16_MIN -32768

typedef long int32_t;           /**< 32 bit signed */
#define INT32_MAX  2147483647‬
#define INT32_MIN -2147483648‬

typedef int16_t intptr_t;       /**< signed int matching size of pointer */
#define INTPTR_MAX INT16_MAX
#define INTPTR_MIN INT16_MIN

typedef uint16_t uintptr_t;     /**< unsigned int matching size of pointer */
#define UINTPTR_MAX UINT16_MAX

typedef uint32_t uintmax_t;     /**< largest possible unsigned type */
#define UINTMAX_MAX UINT32_MAX

typedef int32_t intmax_t;       /**< largest possible signed type */
#define INTMAX_MAX INT32_MAX
#define INTMAX_MIN INT32_MIN

/**< Types 'at least' specified size */
typedef uint8_t uint_least8_t;
#define UINT_LEAST8_MAX UINT8_MAX

typedef uint16_t uint_least16_t;
#define UINT_LEAST16_MAX UINT16_MAX

typedef uint32_t uint_least32_t;
#define UINT_LEAST32_MAX UINT32_MAX

typedef int8_t int_least8_t;
#define INT_LEAST8_MAX INT8_MAX
#define INT_LEAST8_MIN INT8_MIN

typedef int16_t int_least16_t;
#define INT_LEAST16_MAX INT16_MAX
#define INT_LEAST16_MIN INT16_MIN

typedef int32_t int_least32_t;
#define INT_LEAST32_MAX INT32_MAX
#define INT_LEAST32_MIN INT32_MIN

/**< 'Fastest' types that hold the amount */
typedef uint8_t uint_fast8_t;
#define UINT_FAST8_MAX UINT8_MAX

typedef uint16_t uint_fast16_t;
#define UINT_FAST16_MAX UINT16_MAX

typedef uint32_t uint_fast32_t;
#define UINT_FAST32_MAX UINT32_MAX

typedef int8_t int_fast8_t;
#define INT_FAST8_MAX INT8_MAX
#define INT_FAST8_MIN INT8_MIN

typedef int16_t int_fast16_t;
#define INT_FAST16_MAX INT16_MAX
#define INT_FAST16_MIN INT16_MIN

typedef int32_t int_fast32_t;
#define INT_FAST32_MAX INT32_MAX
#define INT_FAST32_MIN INT32_MIN

#endif /* STDINT____H */
